@@ -22,7 +22,7 @@ pb = g.xgridpath(b)
 area = (pa.split([splitpa])[0] <<
         p.split([splita, splitb])[1] <<
         pb.split([splitpb])[0].reversed())
-area.append(path.closepath())
+area[-1].close()
 g.stroke(area, [deco.filled([color.gray(0.8)])])
 g.text(g.pos(0.5 * (a + b), 0)[0], 1,
        r"\int_a^b f(x){\rm d}x", [text.halign.center, text.mathmode])
