@@ -11,8 +11,8 @@ horiz = g.plot(graph.data.function("y=0.5*x", points=2))
 g.finish()
 
 # convert paths to normpaths (for efficiency reasons only)
-fline = path.normpath(fline.path)
-horiz = path.normpath(horiz.path)
+fline = fline.path.normpath()
+horiz = horiz.path.normpath()
 # intersect the lines
 splith, splitf = horiz.intersect(fline)
 
@@ -25,7 +25,7 @@ area = area.joined(fline.split([splitf[-2], splitf[-1]])[1])
 area = area.joined(horiz.split([splith[-1]])[1])
 area.append(path.lineto(*g.vpos(1, 0)))
 area.append(path.lineto(*g.vpos(0, 0)))
-area.append(path.closepath())
+area.append(path.closepath()) # not really needed (filling closes automatically)
 
 c = canvas.canvas()
 
